.device-list-cell{padding:5px 5px 5px 15px;border-bottom:1px solid #dcdee2}.spin-icon-load{-webkit-animation:ani-demo-spin 1s linear infinite;animation:ani-demo-spin 1s linear infinite}.ios-split{height:100vh;border-left:1px solid #e8eaec;border-right:1px solid #e8eaec;word-break:break-all}.ios-split-pane{overflow-y:auto;height:100%}.cell-empty{position:absolute;top:40%;left:50%;-webkit-transform:translate(-50%,-50%);transform:translate(-50%,-50%);text-align:center}.device-list{height:30vh}.ios-img{height:calc(70vh - 10px)}.info-header{background-color:#f8f8f9;padding:5px}.ivu-tabs>.ivu-tabs-bar{border-bottom:0;background-color:#f8f8f9;margin-bottom:0;font-weight:bolder}.ivu-tabs>.ivu-tabs-content{height:100%}#app{font-family:Avenir,Helvetica,Arial,sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale;color:#2c3e50}